import { It } from "./expression-predicates";
import { Expressions } from "./expressions";
import { InjectionToken } from "../static.injector/injection_token";
/**
 * A function that accepts a [Proxy](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Proxy)
 * and either plays expected interaction or returns a predicate function.
 * See {@link IMock.setup} function and {@link It} class for more details.
 */
export declare type IExpression<T> = (instance: T) => void | any | It<T>;
/**
 * This class reflects an expression to an expression tree representation.
 */
export interface ExpressionReflector {
    /**
     * Reflects the provided code as an expression tree.
     */
    reflect<T>(expression: IExpression<T>): Expressions<T>[];
}
/**
 * Injection token for an expression reflector
 */
export declare const EXPRESSION_REFLECTOR: InjectionToken<ExpressionReflector>;
/**
 * @hidden
 */
export declare const EXPRESSIONS: InjectionToken<Expressions<unknown>[]>;
/**
 * @hidden
 */
export interface IReturnValueFactory {
    create(): any;
}
/**
 * @hidden
 */
export declare const GET_RETURN_VALUE: InjectionToken<IReturnValueFactory>;
/**
 * @hidden
 */
export declare const APPLY_RETURN_VALUE: InjectionToken<IReturnValueFactory>;
/**
 * @hidden
 */
export declare const CONSTRUCT_RETURN_VALUE: InjectionToken<IReturnValueFactory>;
